<?php

namespace Teknisa\DF\Modules\Saida\Controllers;

use Teknisa\DF\Modules\Saida\Services\Grs15000ExportacaoNfFaturamentoSapService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;

class Grs15000ExportacaoNfFaturamentoSapController extends \Teknisa\DF\Controllers\TeknisaCrud {

    protected $dataSourceName = '';
    protected $dataSourceNameToSave = '';
    protected $parameterBagColumns = array();
    protected $service;

    public function __construct($crudService, $parameterBag, Grs15000ExportacaoNfFaturamentoSapService $service) {
        parent::__construct($crudService, $parameterBag);
        $this->service = $service;
    }

    public function findNotasFiscais(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName      = 'notafiscal_exportacaoFatSap';
            parent::find($request, $response);
            $dataSets = $response->getDataSets();
            if(!empty($dataSets)) {
                $rows    = $dataSets[0]->getRows();
                $dados   = $this->service->aplicarMascaraInscricao($rows,'NRINSJURNF');
                $dataSet = new DataSet("notafiscal_exportacaoFatSap", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function exportarNf(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $this->service->exportarNf($rows);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }   
   
    public function salvarNotaCredito(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->salvarNotaCredito($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }   
    
    public function getFilterExportSap(DTO\Request $request, DTO\Response $response) {
        try {
            $dados = $this->service->getFilterExportSap();
            $dataSet = new DataSet("filter", $dados);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }   

}