<?php

namespace Teknisa\DF\Modules\Saida\Services;

use Teknisa\DF\Modules\Saida\Factories\NotaFiscalDataUpdateFactory;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Teknisa\DF\Services\LancamentoNotafiscalService;
use Financeiro\Entities\Contareceber;
use Doctrine\ORM\EntityManager;

class NotaFiscalDataUpdateService {
    
    protected $NotaFiscalDataUpdateFactory;
    protected $validacoesEstoqueService;
    protected $lancamentoNotafiscalService;
    protected $environment;
    protected $connection;
    
    public function __construct(NotaFiscalDataUpdateFactory $NotaFiscalDataUpdateFactory, ValidacoesEstoqueService $validacoesEstoqueService, LancamentoNotafiscalService $lancamentoNotafiscalService,
                                $environment, EntityManager $entityManager) {
        $this->NotaFiscalDataUpdateFactory = $NotaFiscalDataUpdateFactory;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->lancamentoNotafiscalService = $lancamentoNotafiscalService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function checkDataUpdate($rowsUpdate, $newDate) {
        $organizacao = $this->environment->getNrOrgTrab();
        
        $statusUpdate = array();
        $statusUpdate['notasAtualizadas'] = 0;
        $statusUpdate['notasErros'] = 0;
        
        foreach($rowsUpdate as $row) {
            $beforeDate = $row['DTENTRSAID'];
            
            try {
                $connection = $this->entityManager->getConnection();
                $connection->beginTransaction();
                
                $cabecalho = $row;
                
                $IDNFAJUSTE = $this->NotaFiscalDataUpdateFactory->getIDNFAJUSTE($cabecalho['CDTIPOOPER'], $cabecalho['IDENTRSAIDOP']);
                if(is_null($IDNFAJUSTE['IDNFAJUSTE'])){
                    throw new \Exception;
                }else{
                   $cabecalho['IDNFAJUSTE'] = $IDNFAJUSTE['IDNFAJUSTE'];
                }
                $cabecalho['DTENTRSAID'] = $newDate;
                $filial = $cabecalho['CDFILIAL'];
                $numLancamento = $cabecalho['NRLANCTONF'];
                $forcaFechamento = false;
                
                $vencimentos = $this->lancamentoNotafiscalService->buscaVencimentos($filial, $numLancamento, $cabecalho['IDENTRSAIDOP']);
                
                $this->validaDataLancamento($cabecalho); 
                $this->NotaFiscalDataUpdateFactory->setDataUpdate($cabecalho);
                $this->lancamentoNotafiscalService->fechaNotaFiscal($cabecalho, $vencimentos, $forcaFechamento); 
                
                $connection->commit();
                
                $statusUpdate['notasAtualizadas'] += 1;
            } catch(\Exception $e) {
                $connection->rollback();
                
                $cabecalho['DTENTRSAID'] = $beforeDate;
                $this->NotaFiscalDataUpdateFactory->setDataUpdate($cabecalho);
                
                $statusUpdate['notasErros'] += 1;
            }
        }
        return $statusUpdate;
    }
    
    protected function validaDataLancamento($cabecalho) {
        try {
            $filial = $cabecalho['CDFILIEMIT'];
            $dtLancamento = $cabecalho['DTENTRSAID'];
            $operacao = "S";
            $sistema = "DFS";
            $dtEmissao = $dtLancamento; //Para nota de saída o sistema não trata a dtEmissão.
            
            $this->validacoesEstoqueService->validaDataLancamento(
                $filial, $dtLancamento, $operacao, $sistema, $dtEmissao
            );
            
        } catch (\Exception $e) {
            $statusUpdate['notasErros'] += 1;
        }
    }
}